package fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories;

import fr.itsonus.bousoleplussbackend.domain.quiz.model.Question;
import fr.itsonus.bousoleplussbackend.domain.quiz.spi.QuestionCacheRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.models.PostgresQuestionDto;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class QuestionCacheProxyRepository implements QuestionCacheRepository {

    private final QuestionCachePostgresRepository questionCachePostgresRepository;

    public List<Question> findQuestions(Long axeId, Long bundleId) {
        return questionCachePostgresRepository.findAllByAxeIdAndBundleId(axeId, bundleId)
                .stream()
                .map(PostgresQuestionDto::toQuestion).collect(Collectors.toList());
    }

    public List<Question> findDefaultQuestions() {
        return questionCachePostgresRepository.findAllByBundleIsNull()
                .stream()
                .map(PostgresQuestionDto::toQuestion).collect(Collectors.toList());
    }

    public List<Question> findDefaultQuestions(Long axeId) {
        return questionCachePostgresRepository.findAllByAxeIdAndBundleIsNull(axeId)
                .stream()
                .map(PostgresQuestionDto::toQuestion).collect(Collectors.toList());
    }

    @Override
    public Page<Question> findAllByAxeId(Long axeId, Pageable pageable) {
        var pageResult = questionCachePostgresRepository.findAllByAxeIdAndBundleIsNotNull(axeId, pageable);
        var pageContent = questionCachePostgresRepository.findAllByAxeIdAndBundleIsNotNull(axeId, pageable)
                .stream()
                .map(PostgresQuestionDto::toQuestion).toList();
        return new PageImpl<>(pageContent, pageable, pageResult.getTotalElements());
    }
}
